const LOCALSTORAGE_KEY = 'matheprisma';

/**
 * Liest einen String aus der LocalStorage Datenbank und konvertiert ihn in ein Objekt.
 *
 * @returns {Object}
 */
function readLocalStorage()
{
    var result = localStorage.getItem(LOCALSTORAGE_KEY);

    // Bisher gab es noch keinen Eintrag
    if (result === null) {
        result = '{}';
    }

    try {
        result = JSON.parse(result);
    } catch (e) {
        console.warn('Quiz Punkte konnten nicht gelesen werden!');
        result = {};
    }

    return result;
}

/**
 * Konvertiert und speichert die Quiz Punkte in die LocalStorage Datenbank.
 * @param result
 * @returns {boolean}
 */
function writeLocalStorage(result)
{
    try {
        result = JSON.stringify(result)
    } catch (e) {
        console.error('Quiz Punkte konnten nicht in einen String konvertiert werden!', e);
        return false;
    }

    try {
        localStorage.setItem('matheprisma', result);
    } catch (e) {
        console.error('Quiz Punkte konnten nicht gespeichert werden. Eventuell ist der Speicher voll.', e);
        return false;
    }
}

/**
 * Zurückgegeben wird ein Boolean:
 *  True bei erfolg,
 *  False falls schon Punkte für die ID in dem Kapitel gesetzt wurden.
 *
 * @param {string} kapitel
 * @param {string} id
 * @param {Number} punkte
 * @returns {bool}
 */
function setPoints(kapitel, id, punkte)
{
    console.debug('Speichere', punkte, 'Punkte für Kapitel', kapitel, 'und Quiz', id);

    var result = readLocalStorage();

    // Prüfen ob das Kapitel bereits existiert
    if (!result[kapitel]) {
        result[kapitel] = {};
    }

    if (result[kapitel][id]) {
        console.warn('Für das Kapitel', kapitel, 'und das Quiz', id, 'wurden bereits Punkte vergeben.');
        return false;
    }

    result[kapitel][id] = punkte;

    return writeLocalStorage(result);
}

/**
 * Gibt die Punkte zurück die in dem Kapitel erreicht wurden (Gesamtpunktzahl)
 *
 * @param {string} kapitel
 * @returns {Number}
 */
function getAllPoints(kapitel)
{
    console.debug('Lese Punkte für Kapitel', kapitel, '.');
    var result = readLocalStorage();

    if (!result[kapitel]) {
        console.debug('Keine Punkte für Kapitel', kapitel, 'vergeben.');
        return 0;
    }

    // Addiere alle Punkte aus dem Kapitel
    var points = 0;
    for (var id in result[kapitel]) {
        // JavaScript Objekte haben ein paar Eigenschaften die per Default gesetzt sind,
        // durch diese Prüfung werden diese nicht mit einberechnet
        if (result[kapitel].hasOwnProperty(id)) {
            points += result[kapitel][id];
        }
    }

    return points;
}

/**
 * Gibt die Punkte zurück die für ein Applet in einem Kapitel erreicht wurden.
 *
 * @param {string} kapitel
 * @param {string} id
 * @returns {Number}
 */
function getPoints(kapitel, id)
{
    console.debug('Lese Punkte für Kapitel', kapitel, 'und Quiz', id);
    var result = readLocalStorage();

    if (!result[kapitel]) {
        console.debug('Keine Punkte für Kapitel', kapitel, 'vergeben.');
        return null;
    }

    if (!result[kapitel][id]) {
        console.debug('Keine Punkte für Kapitel', kapitel, ' und Quiz', id, 'vergeben.');
        return null;
    }

    return result[kapitel][id];
}

/**
 * Setzt die Punkte eines Kapitels wieder auf 0 zurück.
 *
 * @param {string} kapitel
 * @returns {void}
 */
function clearPoints(kapitel)
{
    console.debug('Lösche Punkte für Kapitel', kapitel, '.');

    var result = readLocalStorage();

    if (!result[kapitel]) {
        console.warn('Für das Kapitel', kapitel, 'wurden noch keine Punkte gesetzt.');
        return;
    }

    result[kapitel] = {};

    writeLocalStorage(result);
}
